# QGIS Core
from qgis.core import QgsFeature

# QGIS Utils
from qgis.utils import iface

# Python
import re

MOTIFS     = ['^N ', '^E ', '^S ', '^W ', '^North ', '^East ', '^South ', '^West ',
              ' N$', ' E$', ' S$', ' W$', ' North$', ' East$', ' South$', ' West$',
              '^Northeast ', '^Southeast ', '^Northwest ', '^Southwest ',
              ' Rd', ' Road', ' Ln', 'Lane', ' Ave', ' Avenue', ' St', ' Street', ' Hwy', ' Highway',
              ' Blvd', ' Boulevard', ' Ct', ' Court', ' Pl', ' Place', ' Expy', ' Expressway',
              ' Dr', ' Drive', ' Parkway', 'Pkwy']

# Prep the street name data
# 1. Replace eg, US 90 BUS with US-90 BUS (first space after letters becomes dash)
# 2. Remove all MOTIFS from road names
def custom_prep(clean_lyr):
    # Require the hardcoded fields
    fld_names     = clean_lyr.fields().names()
    flds_required = ['RouteID', 'RouteID_2','Road1Alias', 'Road2Alias']
    flag          = set(flds_required).issubset(fld_names)

    if flag:
        clean_feats = clean_lyr.getFeatures()
        features    = []
        for f in clean_feats:
            for i in ['RouteID', 'RouteID_2']:    
                newval = re.sub(r"([A-z])\s([\d])", r"\1-\2", str(f.attribute(i))).upper()
                f.setAttribute(i, newval)
            
            for i in ['Road1Alias', 'Road2Alias']:
                for to_remove in MOTIFS:
                    newval = re.sub(r'\b{}\b'.format(to_remove), '', str(f.attribute(i)))
                    f.setAttribute(i, newval)
            features.append(f)
    else:
        features = clean_lyr.getFeatures()
        iface.messageBar().pushInfo('GetFeats:', 'Skipped custom prep. Required fields missing.')
    
    return features